from .connection import Connection
from .exceptions import BranchDoesNotExist
from .exceptions import BranchExists
from .exceptions import ForkExists
from .exceptions import GiteaException
from .branch import Branch
from .conf import Config
from .conf import Login
from .fork import Fork
from .git import Git
from .pr import PullRequest
from .ssh_key import SSHKey
from .repo import Repo
from .user import User



from ..util.models import *  # pylint: disable=wildcard-import,unused-wildcard-import
from .person_watchlist_package import PersonWatchlistPackage
from .person_watchlist_project import PersonWatchlistProject
from .person_watchlist_request import PersonWatchlistRequest


class PersonWatchlist(XmlModel):
    XML_TAG = "watchlist"

    project_list: Optional[List[PersonWatchlistProject]] = Field(
        xml_name="project",
    )

    package_list: Optional[List[PersonWatchlistPackage]] = Field(
        xml_name="package",
    )

    request_list: Optional[List[PersonWatchlistRequest]] = Field(
        xml_name="request",
    )



import difflib

import osc.commandline
from .. import oscerr


class RepoRemoveCommand(osc.commandline.OscCommand):
    """
    Remove repositories from project meta
    """

    name = "remove"
    aliases = ["rm"]
    parent = "RepoCommand"

    def init_arguments(self):
        self.add_argument(
            "project",
            help="Name of the project",
        )
        self.add_argument(
            "--repo",
            metavar="[NAME]",
            action="append",
            required=True,
            help="Name of the repository we're removing. Can be specified multiple times.",
        )
        self.add_argument(
            "--yes",
            action="store_true",
            help="Proceed without asking",
        )

    def run(self, args):
        from .. import obs_api
        from ..output import get_user_input

        project_obj = obs_api.Project.from_api(args.apiurl, args.project)
        old = project_obj.to_string()

        for repo in args.repo:
            if project_obj.repository_list is not None:
                project_obj.repository_list = [i for i in project_obj.repository_list if i.name != repo]
            if project_obj.publish_list is not None:
                project_obj.publish_list = [
                    i for i in project_obj.publish_list if i.flag != "disable" or i.repository != repo
                ]

        if not project_obj.has_changed():
            return

        if not args.yes:
            new = project_obj.to_string()
            diff = difflib.unified_diff(old.splitlines(), new.splitlines(), fromfile="old", tofile="new")
            print("\n".join(diff))
            print()

            reply = get_user_input(
                f"""
                You're changing meta of project '{args.project}'
                Do you want to apply the changes?
                """,
                answers={"y": "yes", "n": "no"},
            )

            if reply == "n":
                raise oscerr.UserAbort()

        project_obj.to_api(args.apiurl)



from typing import Optional

from .connection import Connection
from .connection import GiteaHTTPResponse
from .exceptions import ForkExists
from .exceptions import GiteaException


class Fork:
    @classmethod
    def list(
        cls,
        conn: Connection,
        owner: str,
        repo: str,
    ) -> GiteaHTTPResponse:
        """
        List forks of a repository.

        :param conn: Gitea ``Connection`` instance.
        :param owner: Owner of the repo.
        :param repo: Name of the repo.
        """

        url = conn.makeurl("repos", owner, repo, "forks")
        return conn.request("GET", url)

    @classmethod
    def create(
        cls,
        conn: Connection,
        owner: str,
        repo: str,
        *,
        new_repo_name: Optional[str] = None,
        target_org: Optional[str] = None,
        exist_ok: bool = False,
    ) -> GiteaHTTPResponse:
        """
        Fork a repository.

        :param conn: Gitea ``Connection`` instance.
        :param owner: Owner of the repo.
        :param repo: Name of the repo.
        :param new_repo_name: Name of the forked repository.
        :param target_org: Name of the organization, if forking into organization.
        :param exist_ok: A ``ForkExists`` exception is raised when the target exists. Set to ``True`` to avoid throwing the exception.
        """

        json_data = {
            "name": new_repo_name,
            "organization": target_org,
        }
        url = conn.makeurl("repos", owner, repo, "forks")
        try:
            return conn.request("POST", url, json_data=json_data)
        except GiteaException as e:
            # use ForkExists exception to parse fork_owner and fork_repo from the response
            if e.status == 409:
                fork_exists_exception = ForkExists(e.response, owner, repo)
                if exist_ok:
                    from . import Repo
                    return Repo.get(conn, fork_exists_exception.fork_owner, fork_exists_exception.fork_repo)
                raise fork_exists_exception from None
            raise



from ..util.models import *  # pylint: disable=wildcard-import,unused-wildcard-import


class Serviceinfo(XmlModel):
    XML_TAG = "serviceinfo"

    xsrcmd5: Optional[str] = Field(
        xml_attribute=True,
    )

    lsrcmd5: Optional[str] = Field(
        xml_attribute=True,
    )

    error: Optional[str] = Field(
        xml_attribute=True,
    )

    code: Optional[str] = Field(
        xml_attribute=True,
    )



import osc.commandline


class SimpleCommand(osc.commandline.OscCommand):
    """
    A command that does nothing

    More description
    of what the command does.
    """

    # command name
    name = "simple"

    # options and positional arguments
    def init_arguments(self):
        self.add_argument(
            "--bool-option",
            action="store_true",
            help="...",
        )
        self.add_argument(
            "arguments",
            metavar="arg",
            nargs="+",
            help="...",
        )

    # code of the command
    def run(self, args):
        print(f"Bool option is {args.bool_option}")
        print(f"Positional arguments are {args.arguments}")



import os
import shutil
import tempfile
import unittest

import osc.conf

from .common import patch


OSCRC = """
[general]
apiurl = https://api.opensuse.org
user = Admin
pass = opensuse
passx = unused
sshkey = ~/.ssh/id_rsa.pub
packagecachedir = /var/tmp/osbuild-packagecache
su-wrapper = sudo
build-cmd = /usr/bin/build
build-type = kvm
build-root = /var/tmp/build-root/%(repo)s-%(arch)s
build-uid = 1000:1000
build-device = /dev/null
build-memory = 1024
build-shell-after-fail = 0
build-swap = /tmp/build-swap
build-vmdisk-rootsize = 10240
build-vmdisk-swapsize = 512
build-vmdisk-filesystem = ext4
build-vm-user = abuild
build-kernel = /boot/vmlinuz
build-initrd = /boot/initrd
download-assets-cmd = /usr/lib/build/download_assets
build-jobs = 4
builtin_signature_check = 1
icecream = 0
ccache = 0
sccache = 0
sccache_uri = file:///var/tmp/osbuild-sccache
buildlog_strip_time = 0
debug = 0
http_debug = 0
http_full_debug = 0
http_retries = 3
quiet = 0
verbose = 0
no_preinstallimage = 0
traceback = 0
post_mortem = 0
use_keyring = 0
cookiejar = ~/.local/state/osc/cookiejar
no_verify = 0
disable_hdrmd5_check = 0
do_package_tracking = 1
extra-pkgs = vim strace
build_repository = openSUSE_Factory
getpac_default_project = openSUSE:Factory
checkout_no_colon = 0
project_separator = :
checkout_rooted = 0
exclude_glob = .osc CVS .svn .* _linkerror *~ #*# *.orig *.bak *.changes.vctmp.*
print_web_links = 0
request_list_days = 0
check_filelist = 1
check_for_request_on_action = 1
submitrequest_on_accept_action = cleanup
request_show_interactive = 0
request_show_source_buildstatus = 0
review_inherit_group = 0
submitrequest_accepted_template = bla bla
submitrequest_declined_template = bla bla
linkcontrol = 0
include_request_from_project = 1
local_service_run = 1
include_files = incl *.incl
exclude_files = excl *.excl
maintained_attribute = OBS:Maintained
maintenance_attribute = OBS:MaintenanceProject
maintained_update_project_attribute = OBS:UpdateProject
show_download_progress = 0
vc-cmd = /usr/lib/build/vc
status_mtime_heuristic = 0
plugin-option = plugin-general-option

[https://api.opensuse.org]
credentials_mgr_class=osc.credentials.PlaintextConfigFileCredentialsManager
user = Admin
pass = opensuse
passx = unused
aliases = obs
http_headers = 
    Authorization: Basic QWRtaW46b3BlbnN1c2U=
    X-Foo: Bar
realname = The Administrator
email = admin@example.com
cafile = /path/to/custom_cacert.pem
capath = /path/to/custom_cacert.d/
sslcertck = 1
trusted_prj = openSUSE:* SUSE:*
downloadurl = http://example.com/
sshkey = ~/.ssh/id_rsa.pub
disable_hdrmd5_check = 0
plugin-option = plugin-host-option
"""


class TestExampleConfig(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="osc_test_")
        self.oscrc = os.path.join(self.tmpdir, "oscrc")
        with open(self.oscrc, "w", encoding="utf-8") as f:
            f.write(OSCRC)
        osc.conf.get_config(override_conffile=self.oscrc)
        self.config = osc.conf.config

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_invalid_attribute(self):
        self.assertRaises(AttributeError, setattr, self.config, "new_attribute", "123")

    def test_apiurl(self):
        self.assertEqual(self.config["apiurl"], "https://api.opensuse.org")

    def test_user(self):
        self.assertEqual(self.config["user"], "Admin")

    def test_pass(self):
        self.assertEqual(self.config["pass"], "opensuse")

    def test_passx(self):
        self.assertEqual(self.config["passx"], "unused")

    def test_sshkey(self):
        self.assertEqual(self.config["sshkey"], "~/.ssh/id_rsa.pub")

    def test_packagecachedir(self):
        self.assertEqual(self.config["packagecachedir"], "/var/tmp/osbuild-packagecache")

    def test_su_wrapper(self):
        self.assertEqual(self.config["su-wrapper"], "sudo")

    def test_build_cmd(self):
        self.assertEqual(self.config["build-cmd"], "/usr/bin/build")

    def test_build_type(self):
        self.assertEqual(self.config["build-type"], "kvm")

    def test_build_root(self):
        self.assertEqual(self.config["build-root"], "/var/tmp/build-root/%(repo)s-%(arch)s")

    def test_build_uid(self):
        self.assertEqual(self.config["build-uid"], "1000:1000")

    def test_build_device(self):
        self.assertEqual(self.config["build-device"], "/dev/null")

    def test_build_memory(self):
        self.assertEqual(self.config["build-memory"], 1024)

    def test_build_shell_after_fail(self):
        self.assertEqual(self.config["build-shell-after-fail"], False)

    def test_build_swap(self):
        self.assertEqual(self.config["build-swap"], "/tmp/build-swap")

    def test_build_vmdisk_rootsize(self):
        self.assertEqual(self.config["build-vmdisk-rootsize"], 10240)

    def test_build_vmdisk_swapsize(self):
        self.assertEqual(self.config["build-vmdisk-swapsize"], 512)

    def test_build_vmdisk_filesystem(self):
        self.assertEqual(self.config["build-vmdisk-filesystem"], "ext4")

    def test_build_vm_user(self):
        self.assertEqual(self.config["build-vm-user"], "abuild")

    def test_build_kernel(self):
        self.assertEqual(self.config["build-kernel"], "/boot/vmlinuz")

    def test_build_initrd(self):
        self.assertEqual(self.config["build-initrd"], "/boot/initrd")

    def test_download_assets_cmd(self):
        self.assertEqual(self.config["download-assets-cmd"], "/usr/lib/build/download_assets")

    def test_build_jobs(self):
        self.assertEqual(self.config["build-jobs"], 4)

    def test_builtin_signature_check(self):
        self.assertEqual(self.config["builtin_signature_check"], True)

    def test_icecream(self):
        self.assertEqual(self.config["icecream"], 0)

    def test_ccache(self):
        self.assertEqual(self.config["ccache"], False)

    def test_sccache(self):
        self.assertEqual(self.config["sccache"], False)

    def test_sccache_uri(self):
        self.assertEqual(self.config["sccache_uri"], "file:///var/tmp/osbuild-sccache")

    def test_buildlog_strip_time(self):
        self.assertEqual(self.config["buildlog_strip_time"], False)

    def test_debug(self):
        self.assertEqual(self.config["debug"], False)

    def test_http_debug(self):
        self.assertEqual(self.config["http_debug"], False)

    def test_http_full_debug(self):
        self.assertEqual(self.config["http_full_debug"], False)

    def test_http_retries(self):
        self.assertEqual(self.config["http_retries"], 3)

    def test_quiet(self):
        self.assertEqual(self.config["quiet"], False)

    def test_verbose(self):
        self.assertEqual(self.config["verbose"], False)

    def test_no_preinstallimage(self):
        self.assertEqual(self.config["no_preinstallimage"], False)

    def test_traceback(self):
        self.assertEqual(self.config["traceback"], False)

    def test_post_mortem(self):
        self.assertEqual(self.config["post_mortem"], False)

    def test_use_keyring(self):
        self.assertEqual(self.config["use_keyring"], False)

    def test_cookiejar(self):
        self.assertEqual(self.config["cookiejar"], "~/.local/state/osc/cookiejar")

    def test_no_verify(self):
        self.assertEqual(self.config["no_verify"], False)

    def test_disable_hdrmd5_check(self):
        self.assertEqual(self.config["disable_hdrmd5_check"], False)

    def test_do_package_tracking(self):
        self.assertEqual(self.config["do_package_tracking"], True)

    def test_extra_pkgs(self):
        self.assertEqual(self.config["extra-pkgs"], ["vim", "strace"])

    def test_build_repository(self):
        self.assertEqual(self.config["build_repository"], "openSUSE_Factory")

    def test_getpac_default_project(self):
        self.assertEqual(self.config["getpac_default_project"], "openSUSE:Factory")

    def test_checkout_no_colon(self):
        self.assertEqual(self.config["checkout_no_colon"], False)

    def test_project_separator(self):
        self.assertEqual(self.config["project_separator"], ":")

    def test_checkout_rooted(self):
        self.assertEqual(self.config["checkout_rooted"], False)

    def test_exclude_glob(self):
        self.assertEqual(
            self.config["exclude_glob"],
            [
                ".osc",
                "CVS",
                ".svn",
                ".*",
                "_linkerror",
                "*~",
                "#*#",
                "*.orig",
                "*.bak",
                "*.changes.vctmp.*",
            ],
        )

    def test_print_web_links(self):
        self.assertEqual(self.config["print_web_links"], False)

    def test_request_list_days(self):
        self.assertEqual(self.config["request_list_days"], 0)

    def test_check_filelist(self):
        self.assertEqual(self.config["check_filelist"], True)

    def test_check_for_request_on_action(self):
        self.assertEqual(self.config["check_for_request_on_action"], True)

    def test_submitrequest_on_accept_action(self):
        self.assertEqual(self.config["submitrequest_on_accept_action"], "cleanup")

    def test_request_show_interactive(self):
        self.assertEqual(self.config["request_show_interactive"], False)

    def test_request_show_source_buildstatus(self):
        self.assertEqual(self.config["request_show_source_buildstatus"], False)

    def test_review_inherit_group(self):
        self.assertEqual(self.config["review_inherit_group"], False)

    def test_submitrequest_accepted_template(self):
        self.assertEqual(self.config["submitrequest_accepted_template"], "bla bla")

    def test_submitrequest_declined_template(self):
        self.assertEqual(self.config["submitrequest_declined_template"], "bla bla")

    def test_linkcontrol(self):
        self.assertEqual(self.config["linkcontrol"], False)

    def test_include_request_from_project(self):
        self.assertEqual(self.config["include_request_from_project"], True)

    def test_local_service_run(self):
        self.assertEqual(self.config["local_service_run"], True)

    def test_exclude_files(self):
        self.assertEqual(self.config["exclude_files"], ["excl", "*.excl"])

    def test_include_files(self):
        self.assertEqual(self.config["include_files"], ["incl", "*.incl"])

    def test_maintained_attribute(self):
        self.assertEqual(self.config["maintained_attribute"], "OBS:Maintained")

    def test_maintenance_attribute(self):
        self.assertEqual(self.config["maintenance_attribute"], "OBS:MaintenanceProject")

    def test_maintained_update_project_attribute(self):
        self.assertEqual(self.config["maintained_update_project_attribute"], "OBS:UpdateProject")

    def test_show_download_progress(self):
        self.assertEqual(self.config["show_download_progress"], False)

    def test_vc_cmd(self):
        self.assertEqual(self.config["vc-cmd"], "/usr/lib/build/vc")

    def test_status_mtime_heuristic(self):
        self.assertEqual(self.config["status_mtime_heuristic"], False)

    def test_host_option_user(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["user"], "Admin")

    def test_host_option_pass(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["pass"], "opensuse")

    def test_host_option_http_headers(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(
            host_options["http_headers"],
            [
                ("Authorization", "Basic QWRtaW46b3BlbnN1c2U="),
                ("X-Foo", "Bar"),
            ],
        )

    def test_host_option_realname(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["realname"], "The Administrator")

    def test_host_option_email(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["email"], "admin@example.com")

    def test_host_option_sslcertck(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["sslcertck"], True)

    def test_host_option_cafile(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["cafile"], "/path/to/custom_cacert.pem")

    def test_host_option_capath(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["capath"], "/path/to/custom_cacert.d/")

    def test_host_option_sshkey(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["sshkey"], "~/.ssh/id_rsa.pub")

    def test_host_option_credentials_mgr_class(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(
            host_options["credentials_mgr_class"],
            "osc.credentials.PlaintextConfigFileCredentialsManager",
        )

    def test_host_option_allow_http(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["allow_http"], False)

    def test_host_option_trusted_prj(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["trusted_prj"], ["openSUSE:*", "SUSE:*"])

    def test_host_option_downloadurl(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["downloadurl"], "http://example.com/")

    def test_host_option_disable_hdrmd5_check(self):
        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["disable_hdrmd5_check"], False)

    def test_extra_fields(self):
        self.assertEqual(self.config["plugin-option"], "plugin-general-option")
        self.assertEqual(self.config._extra_fields, {"plugin-option": "plugin-general-option"})

        self.config["new-option"] = "value"
        self.assertEqual(self.config["new-option"], "value")
        self.assertEqual(self.config._extra_fields, {"plugin-option": "plugin-general-option", "new-option": "value"})

        host_options = self.config["api_host_options"][self.config["apiurl"]]
        self.assertEqual(host_options["plugin-option"], "plugin-host-option")
        self.assertEqual(host_options._extra_fields, {"plugin-option": "plugin-host-option"})

        host_options["new-option"] = "value"
        self.assertEqual(host_options["new-option"], "value")
        self.assertEqual(host_options._extra_fields, {"plugin-option": "plugin-host-option", "new-option": "value"})

    def test_apiurl_aliases(self):
        expected = {"https://api.opensuse.org": "https://api.opensuse.org", "obs": "https://api.opensuse.org"}
        self.assertEqual(self.config.apiurl_aliases, expected)
        self.assertEqual(self.config["apiurl_aliases"], expected)


class TestOverrides(unittest.TestCase):
    def test_verbose(self):
        self.options = osc.conf.Options()
        self.assertEqual(self.options.quiet, False)
        self.assertEqual(self.options.verbose, False)

        self.options.quiet = True
        self.options.verbose = True
        self.assertEqual(self.options.quiet, True)
        # ``verbose`` is forced to ``False`` by the ``quiet`` option
        self.assertEqual(self.options.verbose, False)

        self.options.quiet = False
        self.assertEqual(self.options.quiet, False)
        self.assertEqual(self.options.verbose, True)

    def test_http_debug(self):
        self.options = osc.conf.Options()
        self.assertEqual(self.options.http_debug, False)
        self.assertEqual(self.options.http_full_debug, False)

        self.options.http_full_debug = True
        # ``http_debug`` forced to ``True`` by the ``http_full_debug`` option
        self.assertEqual(self.options.http_debug, True)
        self.assertEqual(self.options.http_full_debug, True)


class TestFromParent(unittest.TestCase):
    def setUp(self):
        self.options = osc.conf.Options()
        self.host_options = osc.conf.HostOptions(apiurl="https://example.com", username="Admin", _parent=self.options)
        self.options.api_host_options[self.host_options.apiurl] = self.host_options

    def test_disable_hdrmd5_check(self):
        self.assertEqual(self.options.disable_hdrmd5_check, False)
        self.assertEqual(self.host_options.disable_hdrmd5_check, False)

        self.options.disable_hdrmd5_check = True

        self.assertEqual(self.options.disable_hdrmd5_check, True)
        self.assertEqual(self.host_options.disable_hdrmd5_check, True)

        self.host_options.disable_hdrmd5_check = False

        self.assertEqual(self.options.disable_hdrmd5_check, True)
        self.assertEqual(self.host_options.disable_hdrmd5_check, False)

    def test_email(self):
        self.assertEqual(self.options.email, None)
        self.assertEqual(self.host_options.email, None)

        self.options.email = "user@example.com"

        self.assertEqual(self.options.email, "user@example.com")
        self.assertEqual(self.host_options.email, "user@example.com")

        self.host_options.email = "another-user@example.com"

        self.assertEqual(self.options.email, "user@example.com")
        self.assertEqual(self.host_options.email, "another-user@example.com")


class TestConf(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="osc_test_")

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def test_write_initial_config(self):
        conffile = os.path.join(self.tmpdir, "oscrc")
        entries = {
            "user": "Admin",
            "pass": "opensuse",
            "apiurl": "https://example.com",
        }
        osc.conf.write_initial_config(conffile, entries)

    def test_api_host_options(self):
        # test that instances do not share any references leaked from the defaults
        conf1 = osc.conf.Options()
        conf2 = osc.conf.Options()

        self.assertEqual(conf1, conf2)  # models are compared by their contents now
        self.assertNotEqual(id(conf1), id(conf2))
        self.assertNotEqual(id(conf1.api_host_options), id(conf2.api_host_options))


class TestCredentialsFromEnv(unittest.TestCase):
    def setUp(self):
        osc.conf.config = None
        self.oscrc = ""

    @patch.dict(os.environ, {"OSC_APIURL": "https://example.com"}, clear=True)
    def test_new_apiurl(self):
        # missing user
        self.assertRaises(
            osc.oscerr.ConfigMissingCredentialsError,
            osc.conf.get_config,
            override_conffile=self.oscrc,
        )

    @patch.dict(
        os.environ,
        {"OSC_APIURL": "https://example.com", "OSC_USERNAME": "user"},
        clear=True,
    )
    def test_new_apiurl_username(self):
        # missing password
        self.assertRaises(
            osc.oscerr.ConfigMissingCredentialsError,
            osc.conf.get_config,
            override_conffile=self.oscrc,
        )

    @patch.dict(
        os.environ,
        {
            "OSC_APIURL": "https://example.com",
            "OSC_USERNAME": "user",
            "OSC_PASSWORD": "secret",
        },
        clear=True,
    )
    def test_new_apiurl_username_password(self):
        # missing password
        osc.conf.get_config(override_conffile=self.oscrc)
        conf = osc.conf.config
        host_options = conf["api_host_options"][conf["apiurl"]]
        self.assertEqual(conf.apiurl, "https://example.com")
        self.assertEqual(host_options.apiurl, "https://example.com")
        self.assertEqual(host_options.username, "user")
        self.assertEqual(host_options.password, "secret")
        self.assertEqual(host_options.credentials_mgr_class, None)

    @patch.dict(
        os.environ,
        {
            "OSC_APIURL": "https://example.com",
            "OSC_USERNAME": "user",
            "OSC_PASSWORD": "secret",
        },
        clear=True,
    )
    def test_new_apiurl_username_password(self):
        # missing password
        osc.conf.get_config(override_conffile=self.oscrc)
        conf = osc.conf.config
        host_options = conf["api_host_options"][conf["apiurl"]]
        self.assertEqual(conf.apiurl, "https://example.com")
        self.assertEqual(host_options.apiurl, "https://example.com")
        self.assertEqual(host_options.username, "user")
        self.assertEqual(host_options.password, "secret")
        self.assertEqual(host_options.credentials_mgr_class, None)

    @patch.dict(
        os.environ,
        {
            "OSC_APIURL": "https://example.com",
            "OSC_USERNAME": "user",
            "OSC_PASSWORD": "secret",
            "OSC_CREDENTIALS_MGR_CLASS": "osc.credentials.PlaintextConfigFileCredentialsManager",
        },
        clear=True,
    )
    def test_new_apiurl_username_password_credmgr(self):
        # missing password
        osc.conf.get_config(override_conffile=self.oscrc)
        conf = osc.conf.config
        host_options = conf["api_host_options"][conf.apiurl]
        self.assertEqual(conf.apiurl, "https://example.com")
        self.assertEqual(host_options.apiurl, "https://example.com")
        self.assertEqual(host_options.username, "user")
        self.assertEqual(host_options.password, "secret")
        self.assertEqual(host_options.credentials_mgr_class, "osc.credentials.PlaintextConfigFileCredentialsManager")


class TestHostOptionsFromEnv(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="osc_test_")
        self.oscrc = os.path.join(self.tmpdir, "oscrc")
        with open(self.oscrc, "w", encoding="utf-8") as f:
            f.write(OSCRC)
        osc.conf.get_config(override_conffile=self.oscrc)
        self.config = osc.conf.config

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    @patch.dict(
        os.environ,
        {
            "OSC_HOST_OBS_USERNAME": "user",
            "OSC_HOST_OBS_PASSWORD": "secret",
            "OSC_HOST_OBS_CREDENTIALS_MGR_CLASS": "osc.credentials.PlaintextConfigFileCredentialsManager",
            "OSC_HOST_OBS_REALNAME": "User",
            "OSC_HOST_OBS_EMAIL": "user@example.com",
        },
        clear=True,
    )
    def test_host_options(self):
        osc.conf.get_config(override_conffile=self.oscrc)
        conf = osc.conf.config
        host_options = conf["api_host_options"][conf["apiurl"]]
        self.assertEqual(conf.apiurl, "https://api.opensuse.org")
        self.assertEqual(host_options.apiurl, "https://api.opensuse.org")
        self.assertEqual(host_options.username, "user")
        self.assertEqual(host_options.password, "secret")
        self.assertEqual(host_options.credentials_mgr_class, "osc.credentials.PlaintextConfigFileCredentialsManager")
        self.assertEqual(host_options.realname, "User")
        self.assertEqual(host_options.email, "user@example.com")

    @patch.dict(
        os.environ,
        {
            "OSC_HOST_OBS_USERNAME": "user",
            "OSC_HOST_OBS_PASSWORD": "secret",
            "OSC_HOST_OBS_CREDENTIALS_MGR_CLASS": "osc.credentials.PlaintextConfigFileCredentialsManager",
            "OSC_HOST_OBS_REALNAME": "User",
            "OSC_HOST_OBS_EMAIL": "user@example.com",
            "OSC_USERNAME": "USER",
            "OSC_PASSWORD": "SECRET",
            "OSC_CREDENTIALS_MGR_CLASS": "osc.credentials.TransientCredentialsManager",
        },
        clear=True,
    )
    def test_host_options_overrides(self):
        # thest if OSC_{USERNAME,PASSWORD,CREDENTIALS_MGR_CLASS} prevail over OSC_HOST_* options
        osc.conf.get_config(override_conffile=self.oscrc)
        conf = osc.conf.config
        host_options = conf["api_host_options"][conf["apiurl"]]
        self.assertEqual(conf.apiurl, "https://api.opensuse.org")
        self.assertEqual(host_options.apiurl, "https://api.opensuse.org")
        self.assertEqual(host_options.username, "USER")
        self.assertEqual(host_options.password, "SECRET")
        self.assertEqual(host_options.credentials_mgr_class, "osc.credentials.TransientCredentialsManager")
        self.assertEqual(host_options.realname, "User")
        self.assertEqual(host_options.email, "user@example.com")


if __name__ == "__main__":
    unittest.main()



import os
import re
import subprocess
from typing import Optional
from typing import Tuple

from .connection import Connection
from .connection import GiteaHTTPResponse
from .user import User


class Repo:
    @classmethod
    def split_id(cls, repo_id: str) -> Tuple[str, str]:
        """
        Split <owner>/<repo> into individual components and return them in a tuple.
        """
        match = re.match(r"^([^/]+)/([^/]+)$", repo_id)
        if not match:
            raise ValueError(f"Invalid repo id: {repo_id}")
        return match.groups()

    @classmethod
    def get(
        cls,
        conn: Connection,
        owner: str,
        repo: str,
    ) -> GiteaHTTPResponse:
        """
        Retrieve details about a repository.

        :param conn: Gitea ``Connection`` instance.
        :param owner: Owner of the repo.
        :param repo: Name of the repo.
        """
        url = conn.makeurl("repos", owner, repo)
        return conn.request("GET", url)

    @classmethod
    def clone(
        cls,
        conn: Connection,
        owner: str,
        repo: str,
        *,
        directory: Optional[str] = None,
        cwd: Optional[str] = None,
        anonymous: bool = False,
        add_remotes: bool = False,
        ssh_private_key_path: Optional[str] = None,
        ssh_strict_host_key_checking: bool = True,
    ) -> str:
        """
        Clone a repository using 'git clone' command, return absolute path to it.

        :param conn: Gitea ``Connection`` instance.
        :param owner: Owner of the repo.
        :param repo: Name of the repo.
        :param directory: The name of a new directory to clone into. Defaults to the repo name.
        :param cwd: Working directory. Defaults to the current working directory.
        :param anonymous: Whether to use``clone_url`` for an anonymous access or use authenticated ``ssh_url``.
        :param add_remotes: Determine and add 'parent' or 'fork' remotes to the cloned repo.
        """
        import shlex

        cwd = os.path.abspath(cwd) if cwd else os.getcwd()
        directory = directory if directory else repo
        # it's perfectly fine to use os.path.join() here because git can take an absolute path
        directory_abspath = os.path.join(cwd, directory)

        repo_data = cls.get(conn, owner, repo).json()
        clone_url = repo_data["clone_url"] if anonymous else repo_data["ssh_url"]

        remotes = {}
        if add_remotes:
            user = User.get(conn).json()
            if repo_data["owner"]["login"] == user["login"]:
                # we're cloning our own repo, setting remote to the parent (if exists)
                parent = repo_data["parent"]
                remotes["parent"] = parent["clone_url"] if anonymous else parent["ssh_url"]
            else:
                # we're cloning someone else's repo, setting remote to our fork (if exists)
                from . import Fork
                forks = Fork.list(conn, owner, repo).json()
                forks = [i for i in forks if i["owner"]["login"] == user["login"]]
                if forks:
                    assert len(forks) == 1
                    fork = forks[0]
                    remotes["fork"] = fork["clone_url"] if anonymous else fork["ssh_url"]

        ssh_args = []
        env = os.environ.copy()

        if ssh_private_key_path:
            ssh_args += [
                # avoid guessing the ssh key, use the specified one
                "-o IdentitiesOnly=yes",
                f"-o IdentityFile={shlex.quote(ssh_private_key_path)}",
            ]

        if not ssh_strict_host_key_checking:
            ssh_args += [
                "-o StrictHostKeyChecking=no",
                "-o UserKnownHostsFile=/dev/null",
                "-o LogLevel=ERROR",
            ]

        if ssh_args:
            env["GIT_SSH_COMMAND"] = f"ssh {' '.join(ssh_args)}"

        # clone
        cmd = ["git", "clone", clone_url, directory]
        subprocess.run(cmd, cwd=cwd, env=env, check=True)

        # setup remotes
        for name, url in remotes.items():
            cmd = ["git", "-C", directory_abspath, "remote", "add", name, url]
            subprocess.run(cmd, cwd=cwd, check=True)

        # store used ssh args (GIT_SSH_COMMAND) in the local git config
        # to allow seamlessly running ``git push`` and other commands
        if ssh_args:
            cmd = ["git", "-C", directory_abspath, "config", "core.sshCommand", f"echo 'Using core.sshCommand: {env['GIT_SSH_COMMAND']}' >&2; {env['GIT_SSH_COMMAND']}"]
            subprocess.run(cmd, cwd=cwd, check=True)

        return directory_abspath



import os
import sys
import unittest

import osc.core
import osc.oscerr

from .common import GET, OscTestCase


FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'update_fixtures')


def suite():
    return unittest.defaultTestLoader.loadTestsFromTestCase(TestUpdate)


class TestUpdate(OscTestCase):
    def _get_fixtures_dir(self):
        return FIXTURES_DIR

    @GET('http://localhost/source/osctest/simple?rev=latest', file='testUpdateNoChanges_files')
    @GET('http://localhost/source/osctest/simple/_meta', file='meta.xml')
    def testUpdateNoChanges(self):
        """update without any changes (the wc is the most recent version)"""
        self._change_to_pkg('simple')
        osc.core.Package('.').update()
        self.assertEqual(sys.stdout.getvalue(), 'At revision 1.\n')

    @GET('http://localhost/source/osctest/simple?rev=2', file='testUpdateNewFile_files')
    @GET('http://localhost/source/osctest/simple/upstream_added?rev=2', file='testUpdateNewFile_upstream_added')
    @GET('http://localhost/source/osctest/simple/_meta', file='meta.xml')
    def testUpdateNewFile(self):
        """a new file was added to the remote package"""
        self._change_to_pkg('simple')
        osc.core.Package('.').update(rev=2)
        exp = 'A    upstream_added\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testUpdateNewFile_files')

    @GET('http://localhost/source/osctest/simple?rev=2', file='testUpdateNewFileLocalExists_files')
    def testUpdateNewFileLocalExists(self):
        """
        a new file was added to the remote package but the same (unversioned)
        file exists locally
        """
        self._change_to_pkg('simple')
        self.assertRaises(osc.oscerr.PackageFileConflict, osc.core.Package('.').update, rev=2)

    @GET('http://localhost/source/osctest/simple?rev=2', file='testUpdateDeletedFile_files')
    @GET('http://localhost/source/osctest/simple/_meta', file='meta.xml')
    def testUpdateDeletedFile(self):
        """a file was deleted from the remote package"""
        self._change_to_pkg('simple')
        osc.core.Package('.').update(rev=2)
        exp = 'D    foo\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testUpdateDeletedFile_files')
        self.assertFalse(os.path.exists('foo'))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'foo')))

    @GET('http://localhost/source/osctest/simple?rev=2', file='testUpdateUpstreamModifiedFile_files')
    @GET('http://localhost/source/osctest/simple/foo?rev=2', file='testUpdateUpstreamModifiedFile_foo')
    @GET('http://localhost/source/osctest/simple/_meta', file='meta.xml')
    def testUpdateUpstreamModifiedFile(self):
        """a file was modified in the remote package (local file isn't modified)"""

        self._change_to_pkg('simple')
        osc.core.Package('.').update(rev=2)
        exp = 'U    foo\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testUpdateUpstreamModifiedFile_files')

    @GET('http://localhost/source/osctest/conflict?rev=2', file='testUpdateConflict_files')
    @GET('http://localhost/source/osctest/conflict/merge?rev=2', file='testUpdateConflict_merge')
    @GET('http://localhost/source/osctest/conflict/_meta', file='meta.xml')
    def testUpdateConflict(self):
        """
        a file was modified in the remote package (local file is also modified
        and a merge isn't possible)
        """
        self._change_to_pkg('conflict')
        osc.core.Package('.').update(rev=2)
        exp = 'C    merge\nAt revision 2.\n'
        self._check_digests('testUpdateConflict_files')
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_conflictlist('merge\n')

    @GET('http://localhost/source/osctest/already_in_conflict?rev=2', file='testUpdateAlreadyInConflict_files')
    @GET('http://localhost/source/osctest/already_in_conflict/merge?rev=2', file='testUpdateAlreadyInConflict_merge')
    @GET('http://localhost/source/osctest/already_in_conflict/_meta', file='meta.xml')
    def testUpdateAlreadyInConflict(self):
        """
        a file was modified in the remote package (the local file is already in conflict)
        """
        self._change_to_pkg('already_in_conflict')
        osc.core.Package('.').update(rev=2)
        exp = 'skipping \'merge\' (this is due to conflicts)\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_conflictlist('merge\n')
        self._check_digests('testUpdateAlreadyInConflict_files')

    @GET('http://localhost/source/osctest/deleted?rev=2', file='testUpdateLocalDeletions_files')
    @GET('http://localhost/source/osctest/deleted/foo?rev=2', file='testUpdateLocalDeletions_foo')
    @GET('http://localhost/source/osctest/deleted/merge?rev=2', file='testUpdateLocalDeletions_merge')
    @GET('http://localhost/source/osctest/deleted/_meta', file='meta.xml')
    def testUpdateLocalDeletions(self):
        """
        the files 'foo' and 'merge' were modified in the remote package
        and marked for deletion in the local wc. Additionally the file
        'merge' was modified in the wc before deletion so the local file
        still exists (and a merge with the remote file is not possible)
        """
        self._change_to_pkg('deleted')
        osc.core.Package('.').update(rev=2)
        exp = 'U    foo\nC    merge\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_deletelist('foo\n')
        self._check_conflictlist('merge\n')
        self.assertFilesEqual('foo', os.path.join('.osc', 'sources', 'foo'))
        self._check_digests('testUpdateLocalDeletions_files')

    @GET('http://localhost/source/osctest/restore?rev=latest', file='testUpdateRestore_files')
    @GET('http://localhost/source/osctest/restore/foo?rev=1', file='testUpdateRestore_foo')
    @GET('http://localhost/source/osctest/restore/_meta', file='meta.xml')
    def testUpdateRestore(self):
        """local file 'foo' was deleted with a non osc command and will be restored"""
        self._change_to_pkg('restore')
        osc.core.Package('.').update()
        exp = 'Restored \'foo\'\nAt revision 1.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testUpdateRestore_files')

    @GET('http://localhost/source/osctest/limitsize?rev=latest', file='testUpdateLimitSizeNoChange_filesremote')
    @GET('http://localhost/source/osctest/limitsize/_meta', file='meta.xml')
    def testUpdateLimitSizeNoChange(self):
        """
        a new file was added to the remote package but isn't checked out because
        of the size constraint
        """
        self._change_to_pkg('limitsize')
        osc.core.Package('.').update(size_limit=50)
        exp = 'D    bigfile\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'bigfile')))
        self.assertFalse(os.path.exists('bigfile'))
        self._check_digests('testUpdateLimitSizeNoChange_files', 'bigfile')

    @GET('http://localhost/source/osctest/limitsize_local?rev=latest', file='testUpdateLocalLimitSizeNoChange_filesremote')
    @GET('http://localhost/source/osctest/limitsize_local/_meta', file='meta.xml')
    def testUpdateLocalLimitSizeNoChange(self):
        """
        a new file was added to the remote package but isn't checked out because
        of the local size constraint
        """
        self._change_to_pkg('limitsize_local')
        p = osc.core.Package('.')
        p.update()
        exp = 'D    bigfile\nD    merge\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'bigfile')))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'merge')))
        self.assertFalse(os.path.exists('bigfile'))
        self._check_digests('testUpdateLocalLimitSizeNoChange_files', 'bigfile', 'merge')
        self._check_status(p, 'bigfile', 'S')
        self._check_status(p, 'merge', 'S')

    @GET('http://localhost/source/osctest/limitsize?rev=latest', file='testUpdateLimitSizeAddDelete_filesremote')
    @GET('http://localhost/source/osctest/limitsize/exists?rev=2', file='testUpdateLimitSizeAddDelete_exists')
    @GET('http://localhost/source/osctest/limitsize/_meta', file='meta.xml')
    def testUpdateLimitSizeAddDelete(self):
        """
        a new file (exists) was added to the remote package with
        size < size_limit and one file (nochange) was deleted from the
        remote package (local file 'nochange' is modified). Additionally
        files which didn't change are removed the local wc due to the
        size constraint.
        """
        self._change_to_pkg('limitsize')
        osc.core.Package('.').update(size_limit=10)
        exp = 'A    exists\nD    bigfile\nD    foo\nD    merge\nD    nochange\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'bigfile')))
        self.assertFalse(os.path.exists('bigfile'))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
        self.assertFalse(os.path.exists('foo'))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'merge')))
        self.assertFalse(os.path.exists('merge'))
        # exists because local version is modified
        self.assertTrue(os.path.exists('nochange'))

        self._check_digests('testUpdateLimitSizeAddDelete_files', 'bigfile', 'foo', 'merge', 'nochange')

    @GET('http://localhost/source/osctest/services?rev=latest', file='testUpdateServiceFilesAddDelete_filesremote')
    @GET('http://localhost/source/osctest/services/bigfile?rev=2', file='testUpdateServiceFilesAddDelete_bigfile')
    @GET('http://localhost/source/osctest/services/_service:bar?rev=2', file='testUpdateServiceFilesAddDelete__service:bar')
    @GET('http://localhost/source/osctest/services/_service:foo?rev=2', file='testUpdateServiceFilesAddDelete__service:foo')
    @GET('http://localhost/source/osctest/services/_meta', file='meta.xml')
    def testUpdateAddDeleteServiceFiles(self):
        """update package with _service:* files"""
        self._change_to_pkg('services')
        osc.core.Package('.').update(service_files=True)
        exp = 'A    bigfile\nD    _service:exists\nA    _service:bar\nA    _service:foo\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', '_service:bar')))
        self.assertFileContentEqual('_service:bar', 'another service\n')
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', '_service:foo')))
        self.assertFileContentEqual('_service:foo', 'small\n')
        self.assertTrue(os.path.exists('_service:exists'))
        self._check_digests('testUpdateServiceFilesAddDelete_files', '_service:foo', '_service:bar')

    @GET('http://localhost/source/osctest/services?rev=latest', file='testUpdateServiceFilesAddDelete_filesremote')
    @GET('http://localhost/source/osctest/services/bigfile?rev=2', file='testUpdateServiceFilesAddDelete_bigfile')
    @GET('http://localhost/source/osctest/services/_meta', file='meta.xml')
    def testUpdateDisableAddDeleteServiceFiles(self):
        """update package with _service:* files (with service_files=False)"""
        self._change_to_pkg('services')
        osc.core.Package('.').update()
        exp = 'A    bigfile\nD    _service:exists\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', '_service:bar')))
        self.assertFalse(os.path.exists('_service:bar'))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', '_service:foo')))
        self.assertFalse(os.path.exists('_service:foo'))
        self.assertTrue(os.path.exists('_service:exists'))
        self._check_digests('testUpdateServiceFilesAddDelete_files', '_service:foo', '_service:bar')

    @GET('http://localhost/source/osctest/metamode?meta=1&rev=latest', file='testUpdateMetaMode_filesremote')
    @GET('http://localhost/source/osctest/metamode/_meta?meta=1&rev=1', file='testUpdateMetaMode__meta')
    def testUpdateMetaMode(self):
        """update package with metamode enabled"""
        self._change_to_pkg('metamode')
        p = osc.core.Package('.')
        p.update()
        exp = 'A    _meta\nD    foo\nD    merge\nD    nochange\nAt revision 1.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self.assertFalse(os.path.exists('foo'))
        self.assertFalse(os.path.exists('merge'))
        self.assertFalse(os.path.exists('nochange'))
        self._check_digests('testUpdateMetaMode_filesremote')
        self._check_status(p, '_meta', ' ')

    @GET('http://localhost/source/osctest/new?rev=latest', file='testUpdateNew_filesremote')
    @GET('http://localhost/source/osctest/new/_meta', file='meta.xml')
    def testUpdateNew(self):
        """update a new (empty) package. The package has no revision."""
        self._change_to_pkg('new')
        p = osc.core.Package('.')
        p.update()
        exp = 'At revision None.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testUpdateNew_filesremote')

    # tests to recover from an aborted/broken update

    @GET('http://localhost/source/osctest/simple/foo?rev=2', file='testUpdateResume_foo')
    @GET('http://localhost/source/osctest/simple/merge?rev=2', file='testUpdateResume_merge')
    @GET('http://localhost/source/osctest/simple/_meta', file='meta.xml')
    @GET('http://localhost/source/osctest/simple?rev=2', file='testUpdateResume_files')
    @GET('http://localhost/source/osctest/simple/_meta', file='meta.xml')
    def testUpdateResume(self):
        """resume an aborted update"""
        self._change_to_pkg('resume')
        osc.core.Package('.').update(rev=2)
        exp = 'resuming broken update...\nU    foo\nU    merge\nAt revision 2.\nAt revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self.assertFalse(os.path.exists(os.path.join('.osc', '_in_update')))
        self._check_digests('testUpdateResume_files')

    @GET('http://localhost/source/osctest/simple/foo?rev=1', file='testUpdateResumeDeletedFile_foo')
    @GET('http://localhost/source/osctest/simple/merge?rev=1', file='testUpdateResumeDeletedFile_merge')
    @GET('http://localhost/source/osctest/simple/_meta', file='meta.xml')
    @GET('http://localhost/source/osctest/simple?rev=1', file='testUpdateResumeDeletedFile_files')
    @GET('http://localhost/source/osctest/simple/_meta', file='meta.xml')
    def testUpdateResumeDeletedFile(self):
        """
        resume an aborted update (the file 'added' was already deleted in the first update
        run). It's marked as deleted again (this is due to an expected issue with the update
        code)
        """
        self._change_to_pkg('resume_deleted')
        osc.core.Package('.').update(rev=1)
        exp = 'resuming broken update...\nD    added\nU    foo\nU    merge\nAt revision 1.\nAt revision 1.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self.assertFalse(os.path.exists(os.path.join('.osc', '_in_update')))
        self.assertFalse(os.path.exists('added'))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'added')))
        self._check_digests('testUpdateResumeDeletedFile_files')


if __name__ == '__main__':
    unittest.main()



import csv
import io
import time

from . import api


class BuildHistory:
    def __init__(
        self,
        apiurl: str,
        project: str,
        package: str,
        repository: str,
        arch: str,
        limit: int = 0,
    ):
        self.apiurl = apiurl
        self.project = project
        self.package = package
        self.repository = repository
        self.arch = arch
        self._limit = int(limit)
        self.entries = self._get_entries()

    def _get_entries(self):
        url_path = [
            "build",
            self.project,
            self.repository,
            self.arch,
            self.package,
            "_history",
        ]
        url_query = {}
        if self._limit and self._limit > 0:
            url_query["limit"] = self._limit

        root = api.get(self.apiurl, url_path, url_query)

        result = []
        nodes = api.find_nodes(root, "buildhistory", "entry")
        for node in nodes:
            item = {
                "rev": node.get("rev"),
                "srcmd5": node.get("srcmd5"),
                "ver_rel": node.get("versrel"),
                "build_count": int(node.get("bcnt")),
                "time": time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(int(node.get("time")))),
            }

            # duration may not always be available
            duration = node.get("duration")
            if duration:
                item["duration"] = int(duration)

            result.append(item)
        return result

    def to_csv(self):
        out = io.StringIO()
        header = ["time", "srcmd5", "rev", "ver_rel", "build_count", "duration"]
        writer = csv.DictWriter(out, fieldnames=header, quoting=csv.QUOTE_ALL)
        writer.writeheader()
        for i in self.entries:
            writer.writerow(i)
        return out.getvalue()

    def to_text_table(self):
        from ..core import build_table

        header = ("TIME", "SRCMD5", "VER-REL.BUILD#", "REV", "DURATION")
        data = []
        for i in self.entries:
            item = (
                i["time"],
                i["srcmd5"],
                f"{i['ver_rel']}.{i['build_count']}",
                i["rev"],
                i.get("duration", ""),
            )
            data.extend(item)

        return "\n".join(build_table(len(header), data, header))



from ..util.models import *  # pylint: disable=wildcard-import,unused-wildcard-import
from .enums import BoolString
from .person_owner import PersonOwner
from .person_watchlist import PersonWatchlist
from .status import Status


class Person(XmlModel):
    XML_TAG = "person"

    login: str = Field(
    )

    email: Optional[str] = Field(
    )

    realname: Optional[str] = Field(
    )

    owner: Optional[PersonOwner] = Field(
    )

    state: Optional[str] = Field(
    )

    globalrole_list: Optional[List[str]] = Field(
        xml_name="globalrole",
    )

    watchlist: Optional[PersonWatchlist] = Field(
    )

    ignore_auth_services: Optional[BoolString] = Field(
    )

    def to_human_readable_string(self) -> str:
        """
        Render the object as a human readable string.
        """
        from ..output import KeyValueTable

        table = KeyValueTable()
        table.add("Login", self.login, color="bold")
        table.add("Real name", self.realname)
        table.add("Email", self.email)
        table.add("State", self.state)
        return f"{table}"

    @classmethod
    def from_api(cls, apiurl: str, username: str):
        url_path = ["person", username]
        url_query = {}
        response = cls.xml_request("GET", apiurl, url_path, url_query)
        return cls.from_file(response, apiurl=apiurl)

    @classmethod
    def search(
        cls,
        apiurl: str,
        login: Optional[str] = None,
        email: Optional[str] = None,
        realname: Optional[str] = None,
        state: Optional[str] = None,
        **kwargs,
    ) -> List["Person"]:
        from ..util.xml import xml_parse
        from ..util.xpath import XPathQuery as Q

        url_path = ["search", "person"]
        url_query = {
            "match": Q(
                login=login,
                email=email,
                realname=realname,
                state=state,
                **kwargs,
            ),
        }
        response = cls.xml_request("GET", apiurl, url_path, url_query)
        root = xml_parse(response).getroot()
        assert root.tag == "collection"
        result = []
        for node in root:
            result.append(cls.from_xml(node, apiurl=apiurl))
        return result

    @classmethod
    def cmd_register(
        cls,
        apiurl: str,
        *,
        login: str,
        realname: str,
        email: str,
        password: str,
        note: Optional[str] = None,
        state: Optional[str] = "confirmed",
    ):
        person = UnregisteredPerson(login=login, realname=realname, email=email, password=password, note=note, state=state)
        url_path = ["person"]
        url_query = {
            "cmd": "register",
        }
        response = cls.xml_request("POST", apiurl, url_path, url_query, data=person.to_string())
        return Status.from_file(response, apiurl=apiurl)


class UnregisteredPerson(XmlModel):
    XML_TAG = "unregisteredperson"

    login: str = Field(
    )

    realname: str = Field(
    )

    email: str = Field(
    )

    password: str = Field(
    )

    note: Optional[str] = Field(
    )

    state: Optional[str] = Field(
    )



import io
import textwrap
import unittest

from osc.util.models import *
from osc.obs_api.simple_flag import SimpleFlag


class TestXmlModel(unittest.TestCase):
    def test_attribute(self):
        class TestModel(XmlModel):
            XML_TAG = "tag"
            value: str = Field(xml_attribute=True)

        m = TestModel(value="FOO")
        self.assertEqual(m.dict(), {"value": "FOO"})
        expected = """<tag value="FOO" />"""
        self.assertEqual(m.to_string(), expected)

        # verify that we can also load the serialized data
        m = TestModel.from_string(expected)
        self.assertEqual(m.to_string(), expected)

    def test_element(self):
        class TestModel(XmlModel):
            XML_TAG = "tag"
            value: str = Field()

        m = TestModel(value="FOO")
        self.assertEqual(m.dict(), {"value": "FOO"})
        expected = textwrap.dedent(
            """
            <tag>
              <value>FOO</value>
            </tag>
            """
        ).strip()
        self.assertEqual(m.to_string(), expected)

        # verify that we can also load the serialized data
        m = TestModel.from_string(expected)
        self.assertEqual(m.to_string(), expected)

    def test_element_list(self):
        class TestModel(XmlModel):
            XML_TAG = "tag"
            value_list: List[str] = Field(xml_name="value")

        m = TestModel(value_list=["FOO", "BAR"])
        self.assertEqual(m.dict(), {"value_list": ["FOO", "BAR"]})
        expected = textwrap.dedent(
            """
            <tag>
              <value>FOO</value>
              <value>BAR</value>
            </tag>
            """
        ).strip()
        self.assertEqual(m.to_string(), expected)

        # verify that we can also load the serialized data
        m = TestModel.from_string(expected)
        self.assertEqual(m.to_string(), expected)

    def test_child_model(self):
        class ChildModel(XmlModel):
            XML_TAG = "child"
            value: str = Field()

        class ParentModel(XmlModel):
            XML_TAG = "parent"
            text: str = Field()
            child: ChildModel = Field()

        m = ParentModel(text="TEXT", child={"value": "FOO"})
        expected = textwrap.dedent(
            """
            <parent>
              <text>TEXT</text>
              <child>
                <value>FOO</value>
              </child>
            </parent>
            """
        ).strip()
        self.assertEqual(m.to_string(), expected)

        # verify that we can also load the serialized data
        m = ParentModel.from_string(expected)
        self.assertEqual(m.to_string(), expected)

    def test_child_model_list(self):
        class ChildModel(XmlModel):
            XML_TAG = "child"
            value: str = Field()

        class ParentModel(XmlModel):
            XML_TAG = "parent"
            text: str = Field()
            child: List[ChildModel] = Field()

        m = ParentModel(text="TEXT", child=[{"value": "FOO"}, {"value": "BAR"}])
        expected = textwrap.dedent(
            """
            <parent>
              <text>TEXT</text>
              <child>
                <value>FOO</value>
              </child>
              <child>
                <value>BAR</value>
              </child>
            </parent>
            """
        ).strip()
        self.assertEqual(m.to_string(), expected)

        # verify that we can also load the serialized data
        m = ParentModel.from_string(expected)
        self.assertEqual(m.to_string(), expected)

    def test_child_model_list_wrapped(self):
        class ChildModel(XmlModel):
            XML_TAG = "child"
            value: str = Field()

        class ParentModel(XmlModel):
            XML_TAG = "parent"
            text: str = Field()
            child: List[ChildModel] = Field(xml_wrapped=True, xml_name="children")

        m = ParentModel(text="TEXT", child=[{"value": "FOO"}, {"value": "BAR"}])
        expected = textwrap.dedent(
            """
            <parent>
              <text>TEXT</text>
              <children>
                <child>
                  <value>FOO</value>
                </child>
                <child>
                  <value>BAR</value>
                </child>
              </children>
            </parent>
            """
        ).strip()
        self.assertEqual(m.to_string(), expected)

        # verify that we can also load the serialized data
        m = ParentModel.from_string(expected)
        self.assertEqual(m.to_string(), expected)

    def test_apiurl(self):
        class ChildModel(XmlModel):
            XML_TAG = "child"
            value: str = Field()

        class ParentModel(XmlModel):
            XML_TAG = "parent"
            text: str = Field()
            child: List[ChildModel] = Field(xml_wrapped=True, xml_name="children")

        # serialize the model and load it with apiurl set
        m = ParentModel(text="TEXT", child=[{"value": "FOO"}, {"value": "BAR"}])
        xml = m.to_string()

        apiurl = "https://api.example.com"

        m = ParentModel.from_string(xml, apiurl=apiurl)
        m.child.append({"value": "BAZ"})

        self.assertEqual(m._apiurl, apiurl)
        self.assertEqual(m.child[0]._apiurl, apiurl)
        self.assertEqual(m.child[1]._apiurl, apiurl)
        self.assertEqual(m.child[2]._apiurl, apiurl)

        # test the same as above but with a file
        f = io.StringIO(xml)

        m = ParentModel.from_file(f, apiurl=apiurl)
        m.child.append({"value": "BAZ"})

        self.assertEqual(m._apiurl, apiurl)
        self.assertEqual(m.child[0]._apiurl, apiurl)
        self.assertEqual(m.child[1]._apiurl, apiurl)
        self.assertEqual(m.child[2]._apiurl, apiurl)

    def test_empty_int_optional(self):
        class TestModel(XmlModel):
            XML_TAG = "model"
            num_attr: Optional[int] = Field(xml_attribute=True)
            num_elem: Optional[int] = Field()

        data = textwrap.dedent(
            """
            <model num_attr="">
              <num_elem> </num_elem>
            </model>
            """
        ).strip()
        m = TestModel.from_string(data)
        self.assertEqual(m.num_attr, None)
        self.assertEqual(m.num_elem, None)

    def test_empty_int(self):
        class TestModel(XmlModel):
            XML_TAG = "model"
            num_attr: int = Field(xml_attribute=True)
            num_elem: int = Field()

        data = textwrap.dedent(
            """
            <model num_attr="">
              <num_elem> </num_elem>
            </model>
            """
        ).strip()
        self.assertRaises(TypeError, TestModel.from_string, data)

    def test_simple_flag(self):
        class TestModel(XmlModel):
            XML_TAG = "model"
            simple_flag: Optional[SimpleFlag] = Field(
                xml_wrapped=True,
            )

        data = textwrap.dedent(
            """
            <model>
              <simple_flag>
                <enable />
              </simple_flag>
            </model>
            """
        ).strip()

        m = TestModel.from_string(data)
        self.assertEqual(m.simple_flag, "enable")
        self.assertEqual(data, m.to_string())


if __name__ == "__main__":
    unittest.main()



import os
import unittest

import osc.core
import osc.oscerr

from .common import OscTestCase


FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'deletefile_fixtures')


def suite():
    return unittest.defaultTestLoader.loadTestsFromTestCase(TestDeleteFiles)


class TestDeleteFiles(OscTestCase):
    def _get_fixtures_dir(self):
        return FIXTURES_DIR

    def testSimpleRemove(self):
        """delete a file ('foo') from the wc"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('foo')
        self.__check_ret(ret, True, ' ')
        self.assertFalse(os.path.exists('foo'))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
        self._check_deletelist('foo\n')
        self._check_status(p, 'foo', 'D')

    def testDeleteModified(self):
        """delete modified file ('nochange') from the wc (without force)"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('nochange')
        self.__check_ret(ret, False, 'M')
        self.assertTrue(os.path.exists('nochange'))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'nochange')))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self._check_status(p, 'nochange', 'M')

    def testDeleteUnversioned(self):
        """delete an unversioned file ('toadd2') from the wc"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('toadd2')
        self.__check_ret(ret, False, '?')
        self.assertTrue(os.path.exists('toadd2'))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self._check_status(p, 'toadd2', '?')

    def testDeleteAdded(self):
        """delete an added file ('toadd1') from the wc (without force)"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('toadd1')
        self.__check_ret(ret, False, 'A')
        self.assertTrue(os.path.exists('toadd1'))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self._check_status(p, 'toadd1', 'A')

    def testDeleteReplaced(self):
        """delete an added file ('merge') from the wc (without force)"""
        self._change_to_pkg('replace')
        p = osc.core.Package('.')
        ret = p.delete_file('merge')
        self.__check_ret(ret, False, 'R')
        self.assertTrue(os.path.exists('merge'))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self._check_addlist('toadd1\nmerge\n')
        self._check_status(p, 'merge', 'R')

    def testDeleteConflict(self):
        """delete a file ('foo', state='C') from the wc (without force)"""
        self._change_to_pkg('conflict')
        p = osc.core.Package('.')
        ret = p.delete_file('foo')
        self.__check_ret(ret, False, 'C')
        self.assertTrue(os.path.exists('foo'))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self._check_conflictlist('foo\n')
        self._check_status(p, 'foo', 'C')

    def testDeleteModifiedForce(self):
        """force deletion modified file ('nochange') from wc"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('nochange', force=True)
        self.__check_ret(ret, True, 'M')
        self.assertFalse(os.path.exists('nochange'))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'nochange')))
        self._check_deletelist('nochange\n')
        self._check_status(p, 'nochange', 'D')

    def testDeleteUnversionedForce(self):
        """delete an unversioned file ('toadd2') from the wc (with force)"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('toadd2', force=True)
        self.__check_ret(ret, True, '?')
        self.assertFalse(os.path.exists('toadd2'))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self.assertRaises(osc.oscerr.OscIOError, p.status, 'toadd2')

    def testDeleteAddedForce(self):
        """delete an added file ('toadd1') from the wc (with force)"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('toadd1', force=True)
        self.__check_ret(ret, True, 'A')
        self.assertFalse(os.path.exists('toadd1'))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
        self.assertRaises(osc.oscerr.OscIOError, p.status, 'toadd1')

    def testDeleteReplacedForce(self):
        """delete an added file ('merge') from the wc (with force)"""
        self._change_to_pkg('replace')
        p = osc.core.Package('.')
        ret = p.delete_file('merge', force=True)
        self.__check_ret(ret, True, 'R')
        self.assertFalse(os.path.exists('merge'))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'merge')))
        self._check_deletelist('merge\n')
        self._check_addlist('toadd1\n')
        self._check_status(p, 'merge', 'D')

    def testDeleteConflictForce(self):
        """delete a file ('foo', state='C') from the wc (with force)"""
        self._change_to_pkg('conflict')
        p = osc.core.Package('.')
        ret = p.delete_file('foo', force=True)
        self.__check_ret(ret, True, 'C')
        self.assertFalse(os.path.exists('foo'))
        self.assertTrue(os.path.exists('foo.r2'))
        self.assertTrue(os.path.exists('foo.mine'))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
        self._check_deletelist('foo\n')
        self.assertFalse(os.path.exists(os.path.join('.osc', '_in_conflict')))
        self._check_status(p, 'foo', 'D')

    def testDeleteMultiple(self):
        """delete mutliple files from the wc"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('foo')
        self.__check_ret(ret, True, ' ')
        ret = p.delete_file('merge')
        self.__check_ret(ret, True, ' ')
        self.assertFalse(os.path.exists('foo'))
        self.assertFalse(os.path.exists('merge'))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'merge')))
        self._check_deletelist('foo\nmerge\n')

    def testDeleteAlreadyDeleted(self):
        """delete already deleted file from the wc"""
        self._change_to_pkg('already_deleted')
        p = osc.core.Package('.')
        ret = p.delete_file('foo')
        self.__check_ret(ret, True, 'D')
        self.assertFalse(os.path.exists('foo'))
        self.assertTrue(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
        self._check_deletelist('foo\n')
        self._check_status(p, 'foo', 'D')

    def testDeleteAddedMissing(self):
        """
        delete a file which was added to the wc and is removed again
        (via a non osc command). It's current state is '!'
        """
        self._change_to_pkg('delete')
        p = osc.core.Package('.')
        ret = p.delete_file('toadd1')
        self.__check_ret(ret, True, '!')
        self.assertFalse(os.path.exists('toadd1'))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'toadd1')))
        self._check_deletelist('foo\n')
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))

    def testDeleteSkippedLocalNotExistent(self):
        """
        delete a skipped file: no local file with that name exists
        """
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('skipped')
        self.__check_ret(ret, False, 'S')
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))

    def testDeleteSkippedLocalExistent(self):
        """
        delete a skipped file: a local file with that name exists and will be deleted
        (for instance _service:* files have status 'S' but a local files might exist)
        """
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        ret = p.delete_file('skipped_exists')
        self.__check_ret(ret, True, 'S')
        self.assertFalse(os.path.exists('skipped_exists'))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))

    def __check_ret(self, ret, exp1, exp2):
        self.assertTrue(len(ret) == 2)
        self.assertTrue(ret[0] == exp1)
        self.assertTrue(ret[1] == exp2)


if __name__ == '__main__':
    unittest.main()



import os
import sys
import unittest
from urllib.error import HTTPError
from xml.etree import ElementTree as ET

import osc.core
import osc.oscerr

from .common import GET, PUT, POST, DELETE, OscTestCase


FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'commit_fixtures')


def suite():
    return unittest.defaultTestLoader.loadTestsFromTestCase(TestCommit)


rev_dummy = '<revision rev="repository">\n  <srcmd5>empty</srcmd5>\n</revision>'


class TestCommit(OscTestCase):
    def _get_fixtures_dir(self):
        return FIXTURES_DIR

    @GET('http://localhost/source/osctest/simple?rev=latest', file='testSimple_filesremote')
    @POST('http://localhost/source/osctest/simple?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/simple?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testSimple_missingfilelist', expfile='testSimple_lfilelist')
    @PUT('http://localhost/source/osctest/simple/nochange?rev=repository',
         exp='This file didn\'t change but\nis modified.\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/simple?comment=&cmd=commitfilelist&user=Admin',
          file='testSimple_cfilesremote', expfile='testSimple_lfilelist')
    @GET('http://localhost/request?view=collection&project=osctest&package=simple&states=new%2Creview', file='testOpenRequests')
    def test_simple(self):
        """a simple commit (only one modified file)"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        p.commit()
        exp = 'Sending    nochange\nTransmitting file data .\nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testSimple_cfilesremote')
        self.assertTrue(os.path.exists('nochange'))
        self.assertFilesEqual('nochange', os.path.join('.osc', 'sources', 'nochange'))
        self._check_status(p, 'nochange', ' ')
        self._check_status(p, 'foo', ' ')
        self._check_status(p, 'merge', ' ')

    @GET('http://localhost/source/osctest/add?rev=latest', file='testAddfile_filesremote')
    @POST('http://localhost/source/osctest/add?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/add?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testAddfile_missingfilelist', expfile='testAddfile_lfilelist')
    @PUT('http://localhost/source/osctest/add/add?rev=repository',
         exp='added file\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/add?comment=&cmd=commitfilelist&user=Admin',
          file='testAddfile_cfilesremote', expfile='testAddfile_lfilelist')
    @GET('http://localhost/request?view=collection&project=osctest&package=add&states=new%2Creview', file='testOpenRequests')
    def test_addfile(self):
        """commit a new file"""
        self._change_to_pkg('add')
        p = osc.core.Package('.')
        p.commit()
        exp = 'Sending    add\nTransmitting file data .\nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testAddfile_cfilesremote')
        self.assertTrue(os.path.exists('add'))
        self.assertFilesEqual('add', os.path.join('.osc', 'sources', 'add'))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
        self._check_status(p, 'add', ' ')
        self._check_status(p, 'foo', ' ')
        self._check_status(p, 'merge', ' ')
        self._check_status(p, 'nochange', ' ')

    @GET('http://localhost/source/osctest/delete?rev=latest', file='testDeletefile_filesremote')
    @POST('http://localhost/source/osctest/delete?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/delete?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testDeletefile_cfilesremote', expfile='testDeletefile_lfilelist')
    @GET('http://localhost/request?view=collection&project=osctest&package=delete&states=new%2Creview', file='testOpenRequests')
    def test_deletefile(self):
        """delete a file"""
        self._change_to_pkg('delete')
        p = osc.core.Package('.')
        p.commit()
        exp = 'Deleting    nochange\nTransmitting file data \nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testDeletefile_cfilesremote')
        self.assertFalse(os.path.exists('nochange'))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'nochange')))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self._check_status(p, 'foo', ' ')
        self._check_status(p, 'merge', ' ')

    @GET('http://localhost/source/osctest/conflict?rev=latest', file='testConflictfile_filesremote')
    @POST('http://localhost/source/osctest/conflict?cmd=getprojectservices',
          exp='', text='<services />')
    def test_conflictfile(self):
        """package has a file which is in conflict state"""
        self._change_to_pkg('conflict')
        ret = osc.core.Package('.').commit()
        self.assertTrue(ret == 1)
        exp = 'Please resolve all conflicts before committing using "osc resolved FILE"!\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testConflictfile_filesremote')
        self._check_conflictlist('merge\n')

    @GET('http://localhost/source/osctest/nochanges?rev=latest', file='testNoChanges_filesremote')
    @POST('http://localhost/source/osctest/nochanges?cmd=getprojectservices',
          exp='', text='<services />')
    def test_nochanges(self):
        """package has no changes (which can be committed)"""
        self._change_to_pkg('nochanges')
        p = osc.core.Package('.')
        ret = p.commit()
        self.assertTrue(ret == 1)
        exp = 'nothing to do for package nochanges\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_status(p, 'foo', 'S')
        self._check_status(p, 'merge', '!')
        self._check_status(p, 'nochange', ' ')

    @GET('http://localhost/source/osctest/multiple?rev=latest', file='testMultiple_filesremote')
    @POST('http://localhost/source/osctest/multiple?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/multiple?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testMultiple_missingfilelist', expfile='testMultiple_lfilelist')
    @PUT('http://localhost/source/osctest/multiple/nochange?rev=repository', exp='This file did change.\n', text=rev_dummy)
    @PUT('http://localhost/source/osctest/multiple/add?rev=repository', exp='added file\n', text=rev_dummy)
    @PUT('http://localhost/source/osctest/multiple/add2?rev=repository', exp='add2\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/multiple?comment=&cmd=commitfilelist&user=Admin',
          file='testMultiple_cfilesremote', expfile='testMultiple_lfilelist')
    @GET('http://localhost/request?view=collection&project=osctest&package=multiple&states=new%2Creview', file='testOpenRequests')
    def test_multiple(self):
        """a simple commit (only one modified file)"""
        self._change_to_pkg('multiple')
        p = osc.core.Package('.')
        p.commit()
        exp = 'Deleting    foo\nDeleting    merge\nSending    nochange\n' \
            'Sending    add\nSending    add2\nTransmitting file data ...\nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testMultiple_cfilesremote')
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'merge')))
        self.assertRaises(osc.oscerr.OscIOError, p.status, 'foo')
        self.assertRaises(osc.oscerr.OscIOError, p.status, 'merge')
        self._check_status(p, 'add', ' ')
        self._check_status(p, 'add2', ' ')
        self._check_status(p, 'nochange', ' ')

    @GET('http://localhost/source/osctest/multiple?rev=latest', file='testPartial_filesremote')
    @POST('http://localhost/source/osctest/multiple?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/multiple?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testPartial_missingfilelist', expfile='testPartial_lfilelist')
    @PUT('http://localhost/source/osctest/multiple/add?rev=repository', exp='added file\n', text=rev_dummy)
    @PUT('http://localhost/source/osctest/multiple/nochange?rev=repository', exp='This file did change.\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/multiple?comment=&cmd=commitfilelist&user=Admin',
          file='testPartial_cfilesremote', expfile='testPartial_lfilelist')
    @GET('http://localhost/request?view=collection&project=osctest&package=multiple&states=new%2Creview', file='testOpenRequests')
    def test_partial(self):
        """commit only some files"""
        self._change_to_pkg('multiple')
        p = osc.core.Package('.')
        p.todo = ['foo', 'add', 'nochange']
        p.commit()
        exp = 'Deleting    foo\nSending    nochange\n' \
            'Sending    add\nTransmitting file data ..\nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testPartial_cfilesremote')
        self._check_addlist('add2\n')
        self._check_deletelist('merge\n')
        self._check_status(p, 'add2', 'A')
        self._check_status(p, 'merge', 'D')
        self._check_status(p, 'add', ' ')
        self._check_status(p, 'nochange', ' ')
        self.assertRaises(osc.oscerr.OscIOError, p.status, 'foo')

    @GET('http://localhost/source/osctest/simple?rev=latest', file='testSimple_filesremote')
    @POST('http://localhost/source/osctest/simple?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/simple?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testSimple_missingfilelist', expfile='testSimple_lfilelist')
    @PUT('http://localhost/source/osctest/simple/nochange?rev=repository', exp='This file didn\'t change but\nis modified.\n',
         exception=IOError('test exception'), text=rev_dummy)
    def test_interrupt(self):
        """interrupt a commit"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        self.assertRaises(IOError, p.commit)
        exp = 'Sending    nochange\nTransmitting file data .'
        self.assertTrue(sys.stdout.getvalue(), exp)
        self._check_digests('testSimple_filesremote')
        self.assertTrue(os.path.exists('nochange'))
        self._check_status(p, 'nochange', 'M')

    @GET('http://localhost/source/osctest/allstates?rev=latest', file='testPartial_filesremote')
    @POST('http://localhost/source/osctest/allstates?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/allstates?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testAllStates_missingfilelist', expfile='testAllStates_lfilelist')
    @PUT('http://localhost/source/osctest/allstates/add?rev=repository', exp='added file\n', text=rev_dummy)
    @PUT('http://localhost/source/osctest/allstates/missing?rev=repository', exp='replaced\n', text=rev_dummy)
    @PUT('http://localhost/source/osctest/allstates/nochange?rev=repository', exp='This file did change.\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/allstates?comment=&cmd=commitfilelist&user=Admin',
          file='testAllStates_cfilesremote', expfile='testAllStates_lfilelist')
    @GET('http://localhost/request?view=collection&project=osctest&package=allstates&states=new%2Creview', file='testOpenRequests')
    def test_allstates(self):
        """commit all files (all states are available except 'C')"""
        self._change_to_pkg('allstates')
        p = osc.core.Package('.')
        p.commit()
        exp = 'Deleting    foo\nSending    missing\nSending    nochange\n' \
            'Sending    add\nTransmitting file data ...\nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testAllStates_expfiles', 'skipped')
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
        self.assertFalse(os.path.exists('foo'))
        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
        self._check_status(p, 'add', ' ')
        self._check_status(p, 'nochange', ' ')
        self._check_status(p, 'merge', '!')
        self._check_status(p, 'missing', ' ')
        self._check_status(p, 'skipped', 'S')
        self._check_status(p, 'test', ' ')

    @GET('http://localhost/source/osctest/add?rev=latest', file='testAddfile_filesremote')
    @POST('http://localhost/source/osctest/add?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/add?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testAddfile_cfilesremote', expfile='testAddfile_lfilelist')
    @GET('http://localhost/request?view=collection&project=osctest&package=add&states=new%2Creview', file='testOpenRequests')
    def test_remoteexists(self):
        """file 'add' should be committed but already exists on the server"""
        self._change_to_pkg('add')
        p = osc.core.Package('.')
        p.commit()
        exp = 'Sending    add\nTransmitting file data \nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testAddfile_cfilesremote')
        self.assertTrue(os.path.exists('add'))
        self.assertFilesEqual('add', os.path.join('.osc', 'sources', 'add'))
        self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
        self._check_status(p, 'add', ' ')
        self._check_status(p, 'foo', ' ')
        self._check_status(p, 'merge', ' ')
        self._check_status(p, 'nochange', ' ')

    @GET('http://localhost/source/osctest/branch?rev=latest', file='testExpand_filesremote')
    @POST('http://localhost/source/osctest/branch?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/branch?comment=&cmd=commitfilelist&user=Admin&withvalidate=1&keeplink=1',
          file='testExpand_missingfilelist', expfile='testExpand_lfilelist')
    @PUT('http://localhost/source/osctest/branch/simple?rev=repository', exp='simple modified file.\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/branch?comment=&cmd=commitfilelist&user=Admin&keeplink=1',
          file='testExpand_cfilesremote', expfile='testExpand_lfilelist')
    @GET('http://localhost/source/osctest/branch?rev=87ea02aede261b0267aabaa97c756e7a', file='testExpand_expandedfilesremote')
    @GET('http://localhost/request?view=collection&project=osctest&package=branch&states=new%2Creview', file='testOpenRequests')
    def test_expand(self):
        """commit an expanded package"""
        self._change_to_pkg('branch')
        p = osc.core.Package('.')
        p.commit()
        exp = 'Sending    simple\nTransmitting file data .\nCommitted revision 7.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testExpand_expandedfilesremote')
        self._check_status(p, 'simple', ' ')

    @GET('http://localhost/source/osctest/added_missing?rev=latest', file='testAddedMissing_filesremote')
    @POST('http://localhost/source/osctest/added_missing?cmd=getprojectservices',
          exp='', text='<services />')
    def test_added_missing(self):
        """commit an added file which is missing"""
        self._change_to_pkg('added_missing')
        p = osc.core.Package('.')
        ret = p.commit()
        self.assertTrue(ret == 1)
        exp = 'file \'add\' is marked as \'A\' but does not exist\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_status(p, 'add', '!')

    @GET('http://localhost/source/osctest/added_missing?rev=latest', file='testAddedMissing_filesremote')
    @POST('http://localhost/source/osctest/added_missing?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/added_missing?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testAddedMissing_missingfilelist', expfile='testAddedMissing_lfilelist')
    @PUT('http://localhost/source/osctest/added_missing/bar?rev=repository', exp='foobar\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/added_missing?comment=&cmd=commitfilelist&user=Admin',
          file='testAddedMissing_cfilesremote', expfile='testAddedMissing_lfilelist')
    @GET('http://localhost/request?view=collection&project=osctest&package=added_missing&states=new%2Creview', file='testOpenRequests')
    def test_added_missing2(self):
        """commit an added file, another added file missing (but it's not part of the commit)"""
        self._change_to_pkg('added_missing')
        p = osc.core.Package('.')
        p.todo = ['bar']
        p.commit()
        exp = 'Sending    bar\nTransmitting file data .\nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_status(p, 'add', '!')
        self._check_status(p, 'bar', ' ')

    @GET('http://localhost/source/osctest/simple?rev=latest', file='testSimple_filesremote')
    @POST('http://localhost/source/osctest/simple?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/simple?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testSimple_missingfilelist', expfile='testSimple_lfilelist')
    @PUT('http://localhost/source/osctest/simple/nochange?rev=repository',
         exp='This file didn\'t change but\nis modified.\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/simple?comment=&cmd=commitfilelist&user=Admin',
          expfile='testSimple_lfilelist', text='an error occured', code=500)
    def test_commitfilelist_error(self):
        """commit modified file but when committing the filelist the server returns status 500 (see ticket #65)"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        self._check_status(p, 'nochange', 'M')
        self.assertRaises(HTTPError, p.commit)
        exp = 'Sending    nochange\nTransmitting file data .'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_status(p, 'nochange', 'M')

    @GET('http://localhost/source/osctest/simple?rev=latest', file='testSimple_filesremote')
    @POST('http://localhost/source/osctest/simple?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/simple?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testSimple_missingfilelistwithSHA', expfile='testSimple_lfilelist')
    @POST('http://localhost/source/osctest/simple?comment=&cmd=commitfilelist&user=Admin',
          file='testSimple_missingfilelistwithSHAsum', expfile='testSimple_lfilelistwithSHA')
    @PUT('http://localhost/source/osctest/simple/nochange?rev=repository',
         exp='This file didn\'t change but\nis modified.\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/simple?comment=&cmd=commitfilelist&user=Admin',
          file='testSimple_cfilesremote', expfile='testSimple_lfilelistwithSHA')
    @GET('http://localhost/request?view=collection&project=osctest&package=simple&states=new%2Creview', file='testOpenRequests')
    def test_simple_sha256(self):
        """a simple commit (only one modified file)"""
        self._change_to_pkg('simple')
        p = osc.core.Package('.')
        p.commit()
        exp = 'Sending    nochange\nTransmitting file data .\nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_digests('testSimple_cfilesremote')
        self.assertTrue(os.path.exists('nochange'))
        self.assertFilesEqual('nochange', os.path.join('.osc', 'sources', 'nochange'))
        self._check_status(p, 'nochange', ' ')
        self._check_status(p, 'foo', ' ')
        self._check_status(p, 'merge', ' ')

    @GET('http://localhost/source/osctest/added_missing?rev=latest', file='testAddedMissing_filesremote')
    @POST('http://localhost/source/osctest/added_missing?cmd=getprojectservices',
          exp='', text='<services />')
    @POST('http://localhost/source/osctest/added_missing?comment=&cmd=commitfilelist&user=Admin&withvalidate=1',
          file='testAddedMissing_missingfilelistwithSHA', expfile='testAddedMissing_lfilelist')
    @POST('http://localhost/source/osctest/added_missing?comment=&cmd=commitfilelist&user=Admin',
          file='testAddedMissing_missingfilelistwithSHAsum', expfile='testAddedMissing_lfilelistwithSHA')
    @PUT('http://localhost/source/osctest/added_missing/bar?rev=repository', exp='foobar\n', text=rev_dummy)
    @POST('http://localhost/source/osctest/added_missing?comment=&cmd=commitfilelist&user=Admin',
          file='testAddedMissing_cfilesremote', expfile='testAddedMissing_lfilelistwithSHA')
    @GET('http://localhost/request?view=collection&project=osctest&package=added_missing&states=new%2Creview', file='testOpenRequests')
    def test_added_missing2_sha256(self):
        """commit an added file, another added file missing (but it's not part of the commit)"""
        self._change_to_pkg('added_missing')
        p = osc.core.Package('.')
        p.todo = ['bar']
        p.commit()
        exp = 'Sending    bar\nTransmitting file data .\nCommitted revision 2.\n'
        self.assertEqual(sys.stdout.getvalue(), exp)
        self._check_status(p, 'add', '!')
        self._check_status(p, 'bar', ' ')


if __name__ == '__main__':
    unittest.main()



# This is a private implementation of osc.core that will replace it in the future.
# The existing osc.core needs to stay for a while to emit deprecation warnings.
#
# The cherry-picked imports will be the supported API.

from .api_build import BuildHistory
from .api_configuration import get_configuration_value
from .api_source import add_channels
from .api_source import add_containers
from .api_source import enable_channels
from .api_source import get_linked_packages
from .api_source import release
from .common import print_msg
from .common import format_msg_project_package_options
from .package import ApiPackage
from .package import LocalPackage
from .request import forward_request



import unittest
from typing import Set

from osc.util.models import *
from osc.util.models import get_origin


class TestTyping(unittest.TestCase):
    def test_get_origin_list(self):
        typ = get_origin(list)
        self.assertEqual(typ, None)

    def test_get_origin_list_str(self):
        typ = get_origin(List[str])
        self.assertEqual(typ, list)


class TestNotSet(unittest.TestCase):
    def test_repr(self):
        self.assertEqual(repr(NotSet), "NotSet")

    def test_bool(self):
        self.assertEqual(bool(NotSet), False)


class Test(unittest.TestCase):
    def test_dict(self):
        class TestSubmodel(BaseModel):
            text: str = Field(default="default")

        class TestModel(BaseModel):
            a: str = Field(default="default")
            b: Optional[str] = Field(default=None)
            sub: Optional[List[TestSubmodel]] = Field(default=None)

        m = TestModel()
        self.assertEqual(m.dict(), {"a": "default", "b": None, "sub": None})

        m.b = "B"
        m.sub = [{"text": "one"}, {"text": "two"}]
        self.assertEqual(m.dict(), {"a": "default", "b": "B", "sub": [{"text": "one"}, {"text": "two"}]})

    def test_unknown_fields(self):
        class TestModel(BaseModel):
            pass

        self.assertRaises(TypeError, TestModel, does_not_exist=None)

    def test_uninitialized(self):
        class TestModel(BaseModel):
            field: str = Field()

        self.assertRaises(TypeError, TestModel)

    def test_invalid_type(self):
        class TestModel(BaseModel):
            field: Optional[str] = Field()

        m = TestModel()
        self.assertRaises(TypeError, setattr, m.field, [])

    def test_unsupported_type(self):
        class TestModel(BaseModel):
            field: Set[str] = Field(default=None)

        self.assertRaises(TypeError, TestModel)

    def test_lazy_default(self):
        class TestModel(BaseModel):
            field: List[str] = Field(default=lambda: ["string"])

        m = TestModel()
        self.assertEqual(m.field, ["string"])

    def test_lazy_default_invalid_type(self):
        class TestModel(BaseModel):
            field: List[str] = Field(default=lambda: None)

        self.assertRaises(TypeError, TestModel)

    def test_is_set(self):
        class TestModel(BaseModel):
            field: Optional[str] = Field()

        m = TestModel()

        self.assertNotIn("field", m._values)
        self.assertEqual(m.field, None)

        m.field = "text"

        self.assertIn("field", m._values)
        self.assertEqual(m.field, "text")

    def test_str(self):
        class TestModel(BaseModel):
            field: str = Field(default="default")

        m = TestModel()

        field = m.__fields__["field"]
        self.assertEqual(field.is_model, False)
        self.assertEqual(field.is_optional, False)
        self.assertEqual(field.origin_type, str)

        self.assertEqual(m.field, "default")
        m.field = "text"
        self.assertEqual(m.field, "text")

    def test_optional_str(self):
        class TestModel(BaseModel):
            field: Optional[str] = Field()

        m = TestModel()

        field = m.__fields__["field"]
        self.assertEqual(field.is_model, False)
        self.assertEqual(field.is_optional, True)
        self.assertEqual(field.origin_type, str)

        self.assertEqual(m.field, None)
        m.field = "text"
        self.assertEqual(m.field, "text")

    def test_int(self):
        class TestModel(BaseModel):
            field: int = Field(default=0)

        m = TestModel()

        field = m.__fields__["field"]
        self.assertEqual(field.is_model, False)
        self.assertEqual(field.is_optional, False)
        self.assertEqual(field.origin_type, int)

        self.assertEqual(m.field, 0)
        m.field = 1
        self.assertEqual(m.field, 1)

    def test_optional_int(self):
        class TestModel(BaseModel):
            field: Optional[int] = Field()

        m = TestModel()

        field = m.__fields__["field"]
        self.assertEqual(field.is_model, False)
        self.assertEqual(field.is_optional, True)
        self.assertEqual(field.origin_type, int)

        self.assertEqual(m.field, None)
        m.field = 1
        self.assertEqual(m.field, 1)

    def test_submodel(self):
        class TestSubmodel(BaseModel):
            text: str = Field(default="default")

        class TestModel(BaseModel):
            field: TestSubmodel = Field(default={})

        m = TestModel()

        field = m.__fields__["field"]
        self.assertEqual(field.is_model, True)
        self.assertEqual(field.is_optional, False)
        self.assertEqual(field.origin_type, TestSubmodel)

        m = TestModel(field=TestSubmodel())
        self.assertEqual(m.field.text, "default")

        m = TestModel(field={"text": "text"})
        self.assertEqual(m.field.text, "text")

    def test_optional_submodel(self):
        class TestSubmodel(BaseModel):
            text: str = Field(default="default")

        class TestModel(BaseModel):
            field: Optional[TestSubmodel] = Field(default=None)

        m = TestModel()

        field = m.__fields__["field"]
        self.assertEqual(field.is_model, True)
        self.assertEqual(field.is_optional, True)
        self.assertEqual(field.origin_type, TestSubmodel)
        self.assertEqual(m.field, None)
        m.dict()

        m = TestModel(field=TestSubmodel())
        self.assertIsInstance(m.field, TestSubmodel)
        self.assertEqual(m.field.text, "default")
        m.dict()

        m = TestModel(field={"text": "text"})
        self.assertNotEqual(m.field, None)
        self.assertEqual(m.field.text, "text")
        m.dict()

    def test_list_submodels(self):
        class TestSubmodel(BaseModel):
            text: str = Field(default="default")

        class TestModel(BaseModel):
            field: List[TestSubmodel] = Field(default=[])

        m = TestModel()

        field = m.__fields__["field"]
        self.assertEqual(field.is_model, False)
        self.assertEqual(field.is_model_list, True)
        self.assertEqual(field.is_optional, False)
        self.assertEqual(field.origin_type, list)
        m.dict()

        m = TestModel(field=[TestSubmodel()])
        self.assertEqual(m.field[0].text, "default")
        m.dict()

        m = TestModel(field=[{"text": "text"}])
        self.assertEqual(m.field[0].text, "text")
        m.dict()

        self.assertRaises(TypeError, getattr(m, "field"))

    def test_optional_list_submodels(self):
        class TestSubmodel(BaseModel):
            text: str = Field(default="default")

        class TestModel(BaseModel):
            field: Optional[List[TestSubmodel]] = Field(default=[])

        m = TestModel()

        field = m.__fields__["field"]
        self.assertEqual(field.is_model, False)
        self.assertEqual(field.is_model_list, True)
        self.assertEqual(field.is_optional, True)
        self.assertEqual(field.origin_type, list)
        m.dict()

        m = TestModel(field=[TestSubmodel()])
        self.assertEqual(m.field[0].text, "default")
        m.dict()

        m = TestModel(field=[{"text": "text"}])
        self.assertEqual(m.field[0].text, "text")
        m.dict()

        m.field = None
        self.assertEqual(m.field, None)
        m.dict()

    def test_enum(self):
        class Numbers(Enum):
            one = "one"
            two = "two"

        class TestModel(BaseModel):
            field: Optional[Numbers] = Field(default=None)

        m = TestModel()
        field = m.__fields__["field"]
        self.assertEqual(field.is_model, False)
        self.assertEqual(field.is_optional, True)
        self.assertEqual(field.origin_type, Numbers)
        self.assertEqual(m.field, None)

        m.field = "one"
        self.assertEqual(m.field, "one")

        self.assertRaises(ValueError, setattr, m, "field", "does-not-exist")

    def test_parent(self):
        class ParentModel(BaseModel):
            field: str = Field(default="text")

        class ChildModel(BaseModel):
            field: str = Field(default=FromParent("field"))
            field2: str = Field(default=FromParent("field"))

        p = ParentModel()
        c = ChildModel(_parent=p)
        self.assertEqual(p.field, "text")
        self.assertEqual(c.field, "text")
        self.assertEqual(c.field2, "text")

        c.field = "new-text"
        self.assertEqual(p.field, "text")
        self.assertEqual(c.field, "new-text")
        self.assertEqual(c.field2, "text")

    def test_parent_fallback(self):
        class SubModel(BaseModel):
            field: str = Field(default=FromParent("field", fallback="submodel-fallback"))

        class Model(BaseModel):
            field: str = Field(default=FromParent("field", fallback="model-fallback"))
            sub: Optional[SubModel] = Field()
            sub_list: Optional[List[SubModel]] = Field()

        m = Model()
        s = SubModel(_parent=m)
        m.sub = s
        self.assertEqual(m.field, "model-fallback")
        self.assertEqual(m.sub.field, "model-fallback")

        m = Model(sub={})
        self.assertEqual(m.field, "model-fallback")
        self.assertEqual(m.sub.field, "model-fallback")

        m = Model(sub=SubModel())
        self.assertEqual(m.field, "model-fallback")
        self.assertEqual(m.sub.field, "model-fallback")

        m = Model()
        s = SubModel(_parent=m)
        m.sub_list = [s]
        self.assertEqual(m.field, "model-fallback")
        self.assertEqual(m.sub_list[0].field, "model-fallback")

        m = Model(sub_list=[{}])
        self.assertEqual(m.field, "model-fallback")
        self.assertEqual(m.sub_list[0].field, "model-fallback")

        m = Model(sub_list=[SubModel()])
        self.assertEqual(m.field, "model-fallback")
        self.assertEqual(m.sub_list[0].field, "model-fallback")

        m = Model()
        m.sub_list = []
        m.sub_list.append({})
        self.assertEqual(m.field, "model-fallback")
        self.assertEqual(m.sub_list[0].field, "model-fallback")

    def test_get_callback(self):
        class Model(BaseModel):
            quiet: bool = Field(
                default=False,
            )
            verbose: bool = Field(
                default=False,
                # return False if ``quiet`` is True; return the actual value otherwise
                get_callback=lambda obj, value: False if obj.quiet else value,
            )

        m = Model()
        self.assertEqual(m.quiet, False)
        self.assertEqual(m.verbose, False)

        m.quiet = True
        m.verbose = True
        self.assertEqual(m.quiet, True)
        self.assertEqual(m.verbose, False)

        m.quiet = False
        m.verbose = True
        self.assertEqual(m.quiet, False)
        self.assertEqual(m.verbose, True)

    def test_has_changed(self):
        class TestSubmodel(BaseModel):
            text: str = Field(default="default")

        class TestModel(BaseModel):
            field: Optional[List[TestSubmodel]] = Field(default=[])

        m = TestModel()
        self.assertFalse(m.has_changed())

        # a new instance of empty list
        m.field = []
        self.assertFalse(m.has_changed())

        m.field = [{"text": "one"}, {"text": "two"}]
        self.assertTrue(m.has_changed())

        m.do_snapshot()

        # a new instance of list with new instances of objects with the same data
        m.field = [{"text": "one"}, {"text": "two"}]
        self.assertFalse(m.has_changed())

    def test_append_dict(self):
        class TestSubmodel(BaseModel):
            text: str = Field(default="default")

        class TestModel(BaseModel):
            field: Optional[List[TestSubmodel]] = Field(default=[])

        m = TestModel()
        m.field.append({"text": "value"})
        # dict is converted to object next time the field is retrieved
        self.assertIsInstance(m.field[0], BaseModel)
        self.assertEqual(m.field[0].text, "value")

    def test_ordering(self):
        class TestSubmodel(BaseModel):
            txt: Optional[str] = Field()

        class TestModel(BaseModel):
            num: Optional[int] = Field()
            txt: Optional[str] = Field()
            sub: Optional[TestSubmodel] = Field()
            dct: Optional[Dict[str, TestSubmodel]] = Field()

        m1 = TestModel()
        m2 = TestModel()
        self.assertEqual(m1, m2)

        m1 = TestModel(num=1)
        m2 = TestModel(num=2)
        self.assertNotEqual(m1, m2)
        self.assertLess(m1, m2)
        self.assertGreater(m2, m1)

        m1 = TestModel(txt="a")
        m2 = TestModel(txt="b")
        self.assertNotEqual(m1, m2)
        self.assertLess(m1, m2)
        self.assertGreater(m2, m1)

        m1 = TestModel(sub={})
        m2 = TestModel(sub={})
        self.assertEqual(m1, m2)

        m1 = TestModel(sub={"txt": "a"})
        m2 = TestModel(sub={"txt": "b"})
        self.assertNotEqual(m1, m2)
        self.assertLess(m1, m2)
        self.assertGreater(m2, m1)

        m1 = TestModel(dct={})
        m2 = TestModel(dct={})
        self.assertEqual(m1, m2)

        m1 = TestModel(dct={"a": TestSubmodel()})
        m2 = TestModel(dct={"b": TestSubmodel()})
        self.assertNotEqual(m1, m2)
        self.assertLess(m1, m2)
        self.assertGreater(m2, m1)

        # dict ordering doesn't matter
        m1 = TestModel(dct={"a": TestSubmodel(), "b": TestSubmodel()})
        m2 = TestModel(dct={"b": TestSubmodel(), "a": TestSubmodel()})
        self.assertEqual(m1, m2)


if __name__ == "__main__":
    unittest.main()



import os
import shutil
import subprocess
import tempfile
import unittest

from osc.git_scm.store import GitStore


class TestGitStore(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="osc_test")
        os.chdir(self.tmpdir)
        # 'git init -b <initial-branch>' is not supported on older distros
        subprocess.check_output(["git", "init", "-q"])
        subprocess.check_output(["git", "checkout", "-b", "factory", "-q"])
        subprocess.check_output(["git", "remote", "add", "origin", "https://example.com/packages/my-package.git"])

    def tearDown(self):
        try:
            shutil.rmtree(self.tmpdir)
        except OSError:
            pass

    def test_package(self):
        store = GitStore(self.tmpdir)
        self.assertEqual(store.package, "my-package")

    def test_project(self):
        store = GitStore(self.tmpdir)
        self.assertEqual(store.project, "openSUSE:Factory")

    def test_last_buildroot(self):
        store = GitStore(self.tmpdir)
        self.assertEqual(store.last_buildroot, None)
        store.last_buildroot = ("repo", "arch", "vm_type")

        store = GitStore(self.tmpdir)
        self.assertEqual(store.last_buildroot, ("repo", "arch", "vm_type"))

    def test_last_buildroot_empty_vm_type(self):
        store = GitStore(self.tmpdir)
        self.assertEqual(store.last_buildroot, None)
        store.last_buildroot = ("repo", "arch", None)

        store = GitStore(self.tmpdir)
        self.assertEqual(store.last_buildroot, ("repo", "arch", None))

    def test_scmurl(self):
        store = GitStore(self.tmpdir)
        self.assertEqual(store.scmurl, "https://example.com/packages/my-package.git")


if not shutil.which("git"):
    TestGitStore = unittest.skip("The 'git' executable is not available")(TestGitStore)


if __name__ == "__main__":
    unittest.main()



from ..util.models import *  # pylint: disable=wildcard-import,unused-wildcard-import
from .request_sourcediff_file_diff import RequestSourcediffFileDiff
from .request_sourcediff_file_new import RequestSourcediffFileNew
from .request_sourcediff_file_old import RequestSourcediffFileOld


class RequestSourcediffFilesFile(XmlModel):
    XML_TAG = "file"

    state: str = Field(
        xml_attribute=True,
    )

    old: Optional[RequestSourcediffFileOld] = Field(
    )

    new: Optional[RequestSourcediffFileNew] = Field(
    )

    diff: RequestSourcediffFileDiff = Field(
    )
